/* BDF - Copyright (c) 2015-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.api.BdfExtensionInitializer;
import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerConstants;
import fr.exemole.bdfserver.api.externalsource.FichothequeSharing;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.session.LoginException;
import fr.exemole.bdfserver.api.session.SessionObserver;
import fr.exemole.bdfserver.api.storage.JsAnalyser;
import fr.exemole.bdfserver.api.storage.ResourceStorages;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.api.users.BdfUserConstants;
import fr.exemole.bdfserver.conf.BdfServerDirs;
import fr.exemole.bdfserver.conf.Conf;
import fr.exemole.bdfserver.conf.ConfConstants;
import fr.exemole.bdfserver.conf.ConfUtils;
import fr.exemole.bdfserver.email.SmtpManager;
import fr.exemole.bdfserver.impl.BdfServerImpl;
import fr.exemole.bdfserver.multi.Multi;
import fr.exemole.bdfserver.multi.MultiConf;
import fr.exemole.bdfserver.multi.commands.CreationParameters;
import fr.exemole.bdfserver.storage.directory.bdfdata.BdfdataDirectories;
import fr.exemole.bdfserver.storage.directory.bdfdata.SphereCopyEngine;
import fr.exemole.bdfserver.tools.storage.ResourceJsAnalyser;
import fr.exemole.bdfserver.tools.storage.Storages;
import fr.exemole.bdfserver.tools.zip.BdfServerZip;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.zip.Deflater;
import java.util.zip.ZipOutputStream;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpSessionBindingEvent;
import javax.servlet.http.HttpSessionBindingListener;
import net.fichotheque.Fichotheque;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.utils.SphereUtils;
import net.mapeadores.util.buildinfo.BuildInfo;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.mimetype.MimeTypeResolver;
import net.mapeadores.util.mimetype.MimeTypeUtils;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.servlets.HttpServletRequestMap;


/**
 *
 * @author Vincent Calame
 */
public class BDFWebapps implements Multi {

    public final static String BDFWEBAPPS_KEY = "fr.exemole.BDFWebapps";
    public final static String BDFSESSION_KEY = "fr.exemole.BDFSession";
    public final static String BDFBINDING_KEY = "fr.exemole.BDFBinding";
    private final Conf conf;
    private final ResourceStorages webappsResourceStorages;
    private final JsAnalyser webappsJsAnalyser;
    private final MultiConf multiConf;
    private final List<BdfExtensionInitializer> extensionInitializerList;
    private final SmtpManager smtpManager;
    private final BuildInfo buildInfo;
    private final boolean loginPasswordDisabled;
    private final boolean authentificationSharingAllowed;
    private final Map<String, BDFSession> sessionsMap = new LinkedHashMap<String, BDFSession>();
    private final Map<String, BdfServer> bdfServerMap = new HashMap<String, BdfServer>();
    private final SortedSet<String> existingNameSet;
    private final SortedSet<String> unmodifiableExistingNameSet;
    private final BDFBinding bdfBinding = new BDFBinding();
    private final FichothequeSharing fichothequeSharing;
    private final MimeTypeResolver mimeTypeResolver = MimeTypeUtils.DEFAULT_RESOLVER;
    private final int maxInactiveInterval = 18000;

    BDFWebapps(Conf conf, MultiConf multiConf, SortedSet<String> existingNameSet, List<BdfExtensionInitializer> extensionInitializerList, SmtpManager smtpManager, boolean loginPasswordDisabled, boolean authentificationSharingAllowed, ResourceStorages webappsResourceStorages) {
        this.conf = conf;
        this.multiConf = multiConf;
        this.existingNameSet = existingNameSet;
        this.unmodifiableExistingNameSet = Collections.unmodifiableSortedSet(existingNameSet);
        this.extensionInitializerList = extensionInitializerList;
        this.buildInfo = BDFInit.buildInfo();
        this.smtpManager = smtpManager;
        this.loginPasswordDisabled = loginPasswordDisabled;
        this.authentificationSharingAllowed = authentificationSharingAllowed;
        this.webappsResourceStorages = webappsResourceStorages;
        this.webappsJsAnalyser = new ResourceJsAnalyser(webappsResourceStorages);
        if ((multiConf != null) && (authentificationSharingAllowed)) {
            this.fichothequeSharing = new InternalFichothequeSharing();
        } else {
            this.fichothequeSharing = null;
        }
    }

    public boolean isMultiBdf() {
        return (multiConf != null);
    }

    @Override
    public MultiConf getMultiConf() {
        return multiConf;
    }

    @Override
    public boolean isAuthentificationSharingAllowed() {
        return authentificationSharingAllowed;
    }

    public boolean isLoginPasswordDisabled() {
        return loginPasswordDisabled;
    }


    @Override
    public ResourceStorages getWebappsResourceStorages() {
        return webappsResourceStorages;
    }

    @Override
    public MimeTypeResolver getMimeTypeResolver() {
        return mimeTypeResolver;
    }

    @Override
    public JsAnalyser getWebappsJsAnalyser() {
        return webappsJsAnalyser;
    }

    @Override
    public SortedSet<String> getExistingNameSet() {
        return unmodifiableExistingNameSet;
    }

    @Override
    public boolean isInit(String name) {
        if (multiConf == null) {
            name = ConfConstants.UNIQUE_NAME;
        }
        return (bdfServerMap.get(name) != null);
    }

    @Override
    public BdfServer getBdfServer(String name) {
        if (multiConf == null) {
            name = ConfConstants.UNIQUE_NAME;
        }
        BdfServer bdfServer = bdfServerMap.get(name);
        if (bdfServer != null) {
            return bdfServer;
        }
        if (!existingNameSet.contains(name)) {
            boolean here = false;
            if ((multiConf != null) && (ConfUtils.isValidMultiName(name))) {
                if (ConfUtils.isExistingName(conf, name)) {
                    here = true;
                }
            }
            if (here) {
                name = name.intern();
                existingNameSet.add(name);
            } else {
                return null;
            }
        }
        String intern = name.intern();
        synchronized (intern) {
            bdfServer = bdfServerMap.get(intern);
            if (bdfServer == null) {
                Map<String, Object> contextObjectMap = new HashMap<String, Object>();
                contextObjectMap.put(BdfServerConstants.SMTPMANAGER_CONTEXTOBJECT, smtpManager);
                contextObjectMap.put(BdfServerConstants.SESSIONOBSERVER_CONTEXTOBJECT, new InternalSessionObserver(name));
                contextObjectMap.put(BdfServerConstants.FICHOTHEQUESHARING_CONTEXTOBJECT, fichothequeSharing);
                contextObjectMap.put(BdfServerConstants.MIMETYPERESOLVER_CONTEXTOBJECT, mimeTypeResolver);
                Storages storages = BDFInit.storages(conf.getBdfServerDirs(name), webappsResourceStorages);
                bdfServer = new BdfServerImpl(name, storages, buildInfo, extensionInitializerList, contextObjectMap);
                bdfServerMap.put(intern, bdfServer);
            }
        }
        return bdfServer;
    }

    @Override
    public synchronized void createFichotheque(CreationParameters creationParameters) {
        if (multiConf == null) {
            throw new UnsupportedOperationException("only with multiBdf");
        }
        String newFichothequeName = creationParameters.getNewFichothequeName();
        if (!ConfUtils.isValidMultiName(newFichothequeName)) {
            throw new IllegalArgumentException("wrong name: " + newFichothequeName);
        }
        if (existingNameSet.contains(newFichothequeName)) {
            throw new IllegalArgumentException("existing name: " + newFichothequeName);
        }
        newFichothequeName = newFichothequeName.intern();
        BdfServerDirs dirs = conf.getBdfServerDirs(newFichothequeName);
        switch (creationParameters.getType()) {
            case CreationParameters.NEW_TYPE:
                BdfdataDirectories.writeStartFiles(dirs, creationParameters.startValues(), true);
                break;
            case CreationParameters.NEW_WITH_SPHERES_TYPE:
                BdfdataDirectories.writeStartFiles(dirs, creationParameters.startValues(), false);
                BdfdataDirectories bdfdataDirectories = BdfdataDirectories.build(dirs);
                SphereCopyEngine.run(bdfdataDirectories, creationParameters.getSourceBdfServer());
                break;
            case CreationParameters.DUPLICATE_TYPE:
                try {
                File tempZipFile = File.createTempFile("bdf", "zip");
                BdfServerZip bdfServerZip = new BdfServerZip(creationParameters.getSourceBdfServer(), creationParameters.getSubsetEligibility());
                try (ZipOutputStream os = new ZipOutputStream(new FileOutputStream(tempZipFile))) {
                    os.setLevel(Deflater.NO_COMPRESSION);
                    bdfServerZip.zip(os);
                }
                ConfUtils.copyZippedFiles(dirs, tempZipFile);
                tempZipFile.delete();
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
            break;
        }
        existingNameSet.add(newFichothequeName);
    }

    @Override
    public BuildInfo getBuildInfo() {
        return buildInfo;
    }

    public short checkMultiAdmin(HttpServletRequestMap requestMap) {
        HttpServletRequest request = requestMap.getRequest();
        HttpSession httpSession = request.getSession(false);
        if (httpSession != null) {
            BDFSession bdfSession = (BDFSession) httpSession.getAttribute(BDFSESSION_KEY);
            if (bdfSession != null) {
                if (bdfSession.isMultiAdmin()) {
                    return MultiConf.CONNECTED_STATE;
                }
            }
        }
        short check = multiConf.checkAuthentification(requestMap);
        if (check == MultiConf.CONNECTED_STATE) {
            if (httpSession == null) {
                httpSession = request.getSession();
                httpSession.setMaxInactiveInterval(maxInactiveInterval);
            }
            BDFSession bdfSession = getOrCreateBDFSession(httpSession);
            bdfSession.setMultiAdmin(true);
        }
        return check;
    }

    public BdfUser getBdfUser(BdfServer bdfServer, RequestMap requestMap, HttpServletRequest request) throws LoginException {
        HttpSession session = request.getSession(false);
        if (session != null) {
            BDFSession bdfSession = (BDFSession) session.getAttribute(BDFSESSION_KEY);
            if (bdfSession != null) {
                BdfUser bdfUser = bdfSession.getBdfUser(bdfServer.getName());
                if (bdfUser != null) {
                    bdfUser.putParameter(BdfUserConstants.SESSION_LASTACCESSEDTIME, System.currentTimeMillis());
                    return bdfUser;
                } else if ((multiConf != null) && (authentificationSharingAllowed)) {
                    Redacteur matchingRedacteur = bdfSession.checkMatchingBdfUser(bdfServer);
                    if (matchingRedacteur != null) {
                        bdfUser = createBdfUser(bdfServer, bdfSession, matchingRedacteur);
                        checkBdfUser(bdfUser, requestMap, request, bdfServer.getName());
                        return bdfUser;
                    }
                }
            }
        }
        return createBdfUser(bdfServer, requestMap, request, session);
    }

    public synchronized void invalidateBdfUser(BdfServer bdfServer, BdfUser bdfUser) {
        String name = bdfServer.getName();
        BDFSession userBdfSession = null;
        for (BDFSession bdfSession : sessionsMap.values()) {
            if (bdfSession.containsBdfUser(name, bdfUser)) {
                userBdfSession = bdfSession;
                break;
            }
        }
        if (userBdfSession != null) {
            userBdfSession.removeBdfUser(name);
            if (userBdfSession.isEmpty()) {
                HttpSession httpSession = userBdfSession.getHttpSession();
                sessionsMap.remove(httpSession.getId());
                httpSession.removeAttribute(BDFSESSION_KEY);
                httpSession.removeAttribute(BDFBINDING_KEY);
                httpSession.invalidate();
            }
        }
    }

    public static BDFWebapps build(ServletContext servletContext) {
        SmtpManager smtpManager = BDFInit.smtpManager(servletContext);
        boolean loginPasswordDisabled = BDFInit.loginPasswordDisabled(servletContext);
        boolean authentificationSharingAllowed = BDFInit.authentificationSharingAllowed(servletContext);
        Conf conf = BDFInit.conf(servletContext);
        boolean multiBdf = conf.isMultiBdf();
        List<BdfExtensionInitializer> extensionInitializerList = BDFInit.extensionInitializerList(conf);
        SortedSet<String> existingNameSet;
        if (multiBdf) {
            existingNameSet = ConfUtils.getExistingNameSet(conf);
        } else {
            existingNameSet = new TreeSet<String>();
            existingNameSet.add(ConfConstants.UNIQUE_NAME);
        }
        ResourceStorages webappsResourceStorages = BDFInit.initWebappsResourceStorages(conf, extensionInitializerList);
        MultiConf multiConf = null;
        if (multiBdf) {
            multiConf = MultiConf.build(ConfUtils.getMultiIniFile(conf), webappsResourceStorages);
        }
        return new BDFWebapps(conf, multiConf, existingNameSet, extensionInitializerList, smtpManager, loginPasswordDisabled, authentificationSharingAllowed, webappsResourceStorages);
    }

    private synchronized BdfUser createBdfUser(BdfServer bdfServer, RequestMap requestMap, HttpServletRequest request, HttpSession httpSession) throws LoginException {
        String bdfLogin = requestMap.getParameter(InteractionConstants.BDF_LOGIN_PARAMETER);
        if (bdfLogin == null) {
            bdfLogin = requestMap.getParameter("bdf-idsphere"); //Ancienne version
        }
        if (bdfLogin == null) {
            throw new LoginException(LoginException.UNDEFINED_ERROR);
        }
        String bdfPassword = requestMap.getParameter(InteractionConstants.BDF_PASSWORD_PARAMETER);
        if (bdfPassword == null) {
            throw new LoginException(LoginException.UNDEFINED_ERROR);
        }
        String bdfSphere = requestMap.getParameter(InteractionConstants.BDF_SPHERE_PARAMETER);
        bdfLogin = bdfLogin.trim();
        if ((!bdfLogin.contains("[")) && (bdfSphere != null)) {
            bdfSphere = bdfSphere.trim();
            if (bdfSphere.length() > 0) {
                bdfLogin = bdfLogin + "[" + bdfSphere + "]";
            }
        }
        Redacteur redacteur;
        try {
            redacteur = SphereUtils.parse(bdfServer.getFichotheque(), bdfLogin);
        } catch (SphereUtils.RedacteurLoginException rle) {
            if (rle.getType() == SphereUtils.UNKNOWN_SPHERE) {
                throw new LoginException(LoginException.SPHERE_ERROR);
            } else {
                throw new LoginException(LoginException.LOGIN_ERROR);
            }
        }
        if (redacteur == null) {
            throw new LoginException(LoginException.LOGIN_ERROR);
        }
        if (!redacteur.isActive()) {
            throw new LoginException(LoginException.INACTIF_ERROR);
        }
        if (!loginPasswordDisabled) {
            if (!bdfServer.getPasswordManager().checkPassword(redacteur.getGlobalId(), bdfPassword)) {
                throw new LoginException(LoginException.PASSWORD_ERROR);
            }
        }
        if (httpSession == null) {
            httpSession = request.getSession();
            httpSession.setMaxInactiveInterval(maxInactiveInterval);
        }
        BDFSession bdfSession = getOrCreateBDFSession(httpSession);
        BdfUser bdfUser = createBdfUser(bdfServer, bdfSession, redacteur);
        checkBdfUser(bdfUser, requestMap, request, bdfServer.getName());
        return bdfUser;
    }

    private BdfUser createBdfUser(BdfServer bdfServer, BDFSession bdfSession, Redacteur redacteur) {
        BdfUser bdfUser = bdfServer.createBdfUser(redacteur);
        bdfSession.addBdfUser(bdfServer.getName(), bdfUser);
        bdfServer.updateFicheSelection(bdfUser);
        return bdfUser;
    }

    private void checkBdfUser(BdfUser bdfUser, RequestMap requestMap, HttpServletRequest request, String fichothequeName) {
        long currentTime = System.currentTimeMillis();
        boolean withJavascript = true;
        if (requestMap.isTrue(InteractionConstants.BDF_NOSCRIPT_PARAMETER)) {
            withJavascript = false;
        }
        bdfUser.putParameter(BdfUserConstants.SESSION_CREATIONTIME, currentTime);
        bdfUser.putParameter(BdfUserConstants.SESSION_LASTACCESSEDTIME, currentTime);
        bdfUser.putParameter(BdfUserConstants.HTML_WITHJAVASCRIPT, withJavascript);
        bdfUser.putParameter(BdfUserConstants.SESSION_ROOTURL, getRootUrl(request, fichothequeName));
    }

    private BDFSession getOrCreateBDFSession(HttpSession httpSession) {
        BDFSession bdfSession = (BDFSession) httpSession.getAttribute(BDFSESSION_KEY);
        if (bdfSession == null) {
            bdfSession = new BDFSession(httpSession);
            httpSession.setAttribute(BDFSESSION_KEY, bdfSession);
            sessionsMap.put(httpSession.getId(), bdfSession);
            httpSession.setAttribute(BDFBINDING_KEY, bdfBinding);
        }
        return bdfSession;
    }


    private static String getRootUrl(HttpServletRequest request, String fichothequeName) {
        StringBuilder url = new StringBuilder();
        String scheme = request.getScheme();
        int port = request.getServerPort();
        if (port < 0) {
            port = 80; // Work around java.net.URL bug
        }
        url.append(scheme);
        url.append("://");
        url.append(request.getServerName());
        if ((scheme.equals("http") && (port != 80))
                || (scheme.equals("https") && (port != 443))) {
            url.append(':');
            url.append(port);
        }
        url.append(request.getContextPath());
        url.append("/");
        if (!fichothequeName.equals(ConfConstants.UNIQUE_NAME)) {
            url.append(fichothequeName);
            url.append("/");
        }
        return url.toString();
    }


    private class BDFBinding implements HttpSessionBindingListener {

        private BDFBinding() {

        }

        @Override
        public void valueBound(HttpSessionBindingEvent httpSessionEvent) {
        }

        @Override
        public void valueUnbound(HttpSessionBindingEvent httpSessionEvent) {
            HttpSession session = httpSessionEvent.getSession();
            sessionsMap.remove(session.getId());
        }

    }


    private class InternalSessionObserver implements SessionObserver {

        private final String fichothequeName;

        private InternalSessionObserver(String fichothequeName) {
            this.fichothequeName = fichothequeName;
        }

        @Override
        public synchronized BdfUser[] getCurrentBdfUserArray() {
            List<BdfUser> bdfUserList = new ArrayList<BdfUser>();
            for (BDFSession bdfSession : sessionsMap.values()) {
                BdfUser bdfUser = bdfSession.getBdfUser(fichothequeName);
                if (bdfUser != null) {
                    bdfUserList.add(bdfUser);
                }
            }
            return bdfUserList.toArray(new BdfUser[bdfUserList.size()]);
        }

    }


    private class InternalFichothequeSharing implements FichothequeSharing {

        private InternalFichothequeSharing() {

        }

        @Override
        public Fichotheque getFichotheque(String name) {
            BdfServer bdfServer = getBdfServer(name);
            if (bdfServer != null) {
                return bdfServer.getFichotheque();
            } else {
                return null;
            }
        }

    }


}
