/* BdfServer - Copyright (c) 2013-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.importation.engines;

import fr.exemole.bdfserver.api.EditSession;
import java.text.ParseException;
import net.fichotheque.ExistingIdException;
import net.fichotheque.FichothequeEditor;
import net.fichotheque.croisement.CroisementEditor;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.importation.ThesaurusImport;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.ParentRecursivityException;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.thesaurus.ThesaurusEditor;
import net.fichotheque.tools.FichothequeTools;
import net.fichotheque.tools.thesaurus.ThesaurusTools;
import net.fichotheque.utils.FichothequeUtils;
import net.fichotheque.utils.ThesaurusUtils;
import net.mapeadores.util.localisation.Langs;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusImportEngine {

    private ThesaurusImportEngine() {
    }

    public static void runThesaurusImport(EditSession editSession, ExtractionContext extractionContext, ThesaurusImport thesaurusImport) {
        String type = thesaurusImport.getType();
        if (type.equals(ThesaurusImport.CREATION_TYPE)) {
            runCreation(editSession, extractionContext, thesaurusImport);
        } else if (type.equals(ThesaurusImport.CHANGE_TYPE)) {
            runChange(editSession, extractionContext, thesaurusImport);
        } else if (type.equals(ThesaurusImport.REMOVE_TYPE)) {
            runRemove(editSession, thesaurusImport);
        } else if (type.equals(ThesaurusImport.MERGE_TYPE)) {
            runMerge(editSession, thesaurusImport);
        } else if (type.equals(ThesaurusImport.MOVE_TYPE)) {
            runMove(editSession, thesaurusImport);
        }
    }

    private static void runCreation(EditSession editSession, ExtractionContext extractionContext, ThesaurusImport thesaurusImport) {
        Thesaurus thesaurus = thesaurusImport.getThesaurus();
        boolean withIdalpha = thesaurus.isIdalphaType();
        FichothequeEditor fichothequeEditor = editSession.getFichothequeEditor();
        CroisementEditor croisementEditor = fichothequeEditor.getCroisementEditor();
        ThesaurusEditor thesaurusEditor = fichothequeEditor.getThesaurusEditor(thesaurus.getSubsetKey());
        int motcleCount = thesaurusImport.getMotcleImportCount();
        for (int i = 0; i < motcleCount; i++) {
            ThesaurusImport.CreationMotcleImport motcleImport = (ThesaurusImport.CreationMotcleImport) thesaurusImport.getMotcleImport(i);
            String newIdalpha = motcleImport.getNewIdalpha();
            if (withIdalpha) {
                if (newIdalpha == null) {
                    continue;
                }
            } else {
                newIdalpha = null;
            }
            try {
                Motcle motcle = thesaurusEditor.createMotcle(motcleImport.getNewId(), newIdalpha);
                ThesaurusUtils.changeMotcleLabelHolder(thesaurusEditor, motcle, motcleImport.getLabelChange());
                FichothequeUtils.changeAttributes(fichothequeEditor, motcle, motcleImport.getAttributeChange());
                LiensImportEngine.run(croisementEditor, motcle, motcleImport.getLiensImport(), extractionContext);
                Motcle parent = null;
                int parentId = motcleImport.getParentId();
                if (parentId > 0) {
                    parent = thesaurus.getMotcleById(parentId);
                } else if (withIdalpha) {
                    String parentIdalpha = motcleImport.getParentIdalpha();
                    if (parentIdalpha != null) {
                        parent = thesaurus.getMotcleByIdalpha(parentIdalpha);
                    }
                }
                if (parent != null) {
                    try {
                        thesaurusEditor.setParent(motcle, parent);
                    } catch (ParentRecursivityException pre) {
                    }
                }
            } catch (ExistingIdException eie) {
            } catch (ParseException pe) {
            }
        }
    }

    private static void runChange(EditSession editSession, ExtractionContext extractionContext, ThesaurusImport thesaurusImport) {
        Thesaurus thesaurus = thesaurusImport.getThesaurus();
        boolean withIdalpha = thesaurus.isIdalphaType();
        FichothequeEditor fichothequeEditor = editSession.getFichothequeEditor();
        CroisementEditor croisementEditor = fichothequeEditor.getCroisementEditor();
        ThesaurusEditor thesaurusEditor = fichothequeEditor.getThesaurusEditor(thesaurus.getSubsetKey());
        int motcleCount = thesaurusImport.getMotcleImportCount();
        for (int i = 0; i < motcleCount; i++) {
            ThesaurusImport.ChangeMotcleImport motcleImport = (ThesaurusImport.ChangeMotcleImport) thesaurusImport.getMotcleImport(i);
            Motcle motcle = motcleImport.getMotcle();
            ThesaurusUtils.changeMotcleLabelHolder(thesaurusEditor, motcle, motcleImport.getLabelChange());
            FichothequeUtils.changeAttributes(fichothequeEditor, motcle, motcleImport.getAttributeChange());
            LiensImportEngine.run(croisementEditor, motcle, motcleImport.getLiensImport(), extractionContext);
            if (withIdalpha) {
                String newIdalpha = motcleImport.getNewIdalpha();
                if (newIdalpha != null) {
                    try {
                        thesaurusEditor.setIdalpha(motcle, newIdalpha);
                    } catch (ExistingIdException eie) {
                    } catch (ParseException pe) {
                    }
                }
            }
            Object newParent = motcleImport.getParent();
            if (newParent != null) {
                Motcle newMotcleParent = null;
                if (newParent instanceof Motcle) {
                    newMotcleParent = (Motcle) newParent;
                }
                try {
                    thesaurusEditor.setParent(motcle, newMotcleParent);
                } catch (ParentRecursivityException pre) {
                }
            }
        }
    }

    private static void runRemove(EditSession editSession, ThesaurusImport thesaurusImport) {
        FichothequeEditor fichothequeEditor = editSession.getFichothequeEditor();
        ThesaurusEditor thesaurusEditor = fichothequeEditor.getThesaurusEditor(thesaurusImport.getThesaurus().getSubsetKey());
        int motcleCount = thesaurusImport.getMotcleImportCount();
        int resultCount = 0;
        for (int i = 0; i < motcleCount; i++) {
            ThesaurusImport.MotcleImport motcleImport = thesaurusImport.getMotcleImport(i);
            Motcle motcle = motcleImport.getMotcle();
            FichothequeTools.prepareToRemove(fichothequeEditor, motcle);
            boolean done = thesaurusEditor.removeMotcle(motcle);
            if (done) {
                resultCount++;
            }
        }
    }

    private static void runMerge(EditSession editSession, ThesaurusImport thesaurusImport) {
        FichothequeEditor fichothequeEditor = editSession.getFichothequeEditor();
        int motcleCount = thesaurusImport.getMotcleImportCount();
        int resultCount = 0;
        for (int i = 0; i < motcleCount; i++) {
            ThesaurusImport.MergeMotcleImport motcleImport = (ThesaurusImport.MergeMotcleImport) thesaurusImport.getMotcleImport(i);
            try {
                Langs langs = editSession.getBdfServer().getThesaurusLangChecker().getAuthorizedLangs(motcleImport.getDestinationMotcle().getThesaurus());
                ThesaurusTools.merge(fichothequeEditor, motcleImport.getMotcle(), motcleImport.getDestinationMotcle(), langs);
                resultCount++;
            } catch (ParentRecursivityException pre) {
            }
        }
    }

    private static void runMove(EditSession editSession, ThesaurusImport thesaurusImport) {
        FichothequeEditor fichothequeEditor = editSession.getFichothequeEditor();
        Thesaurus destinationThesaurus = thesaurusImport.getDestinationThesaurus();
        Langs langs = editSession.getBdfServer().getThesaurusLangChecker().getAuthorizedLangs(destinationThesaurus);
        int motcleCount = thesaurusImport.getMotcleImportCount();
        for (int i = 0; i < motcleCount; i++) {
            ThesaurusImport.MotcleImport motcleImport = thesaurusImport.getMotcleImport(i);
            ThesaurusTools.move(fichothequeEditor, motcleImport.getMotcle(), destinationThesaurus, langs);
        }
    }

}
