/* BdfServer - Copyright (c) 2013-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.storage;

import fr.exemole.bdfserver.api.storage.ResourceFolder;
import fr.exemole.bdfserver.api.storage.ResourceStorage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;
import net.fichotheque.EditOrigin;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.io.docstream.FileDocStream;
import net.mapeadores.util.mimetype.MimeTypeResolver;
import net.mapeadores.util.mimetype.MimeTypeUtils;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryResourceStorage implements ResourceStorage {


    private final String name;
    private final File rootDirectory;
    private final File backupDirectory;
    private final boolean editable;

    public DirectoryResourceStorage(String name, File rootDirectory, boolean editable, File backupDirectory) {
        this.rootDirectory = rootDirectory;
        this.backupDirectory = backupDirectory;
        this.editable = editable;
        this.name = name;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public boolean containsResource(RelativePath path) {
        File file = getFile(path.toString(), false);
        return (file != null);
    }

    @Override
    public DocStream getResourceDocStream(RelativePath path, MimeTypeResolver mimeTypeResolver) {
        File file = getFile(path.toString(), false);
        if (file == null) {
            return null;
        } else {
            FileDocStream fileDocStream = new FileDocStream(file);
            String mimeType = MimeTypeUtils.getMimeType(mimeTypeResolver, file.getName());
            fileDocStream.setMimeType(mimeType);
            return fileDocStream;
        }
    }

    @Override
    public void saveResource(RelativePath path, InputStream inputStream, EditOrigin editOrigin) throws IOException {
        if (!editable) {
            throw new UnsupportedOperationException();
        }
        File file = getFile(path.toString(), true);
        if (file == null) {
            return;
        }
        if (!StorageUtils.isValidResourceRelativePath(path)) {
            throw new IllegalArgumentException("wrong path (too short or with illegal characters): " + path);
        }
        File backupFile = getBackupFile(path.toString());
        if (backupFile != null) {
            StorageUtils.archive(file, backupFile, editOrigin);
        }
        file.getParentFile().mkdirs();
        try (OutputStream os = new FileOutputStream(file)) {
            IOUtils.copy(inputStream, os);
        }
    }

    @Override
    public boolean removeResource(RelativePath path, EditOrigin editOrigin) {
        if (!editable) {
            throw new UnsupportedOperationException();
        }
        File file = getFile(path.toString(), false);
        if (file == null) {
            return false;
        }
        if (!StorageUtils.isValidResourceRelativePath(path)) {
            throw new IllegalArgumentException("wrong characters in path: " + path);
        }
        file.delete();
        String[] array = path.toArray();
        int pathLength = array.length - 1;
        for (int i = pathLength; i > 0; i--) {
            StringBuilder pathBuf = new StringBuilder();
            for (int j = 0; j < pathLength; j++) {
                pathBuf.append(array[j]);
                pathBuf.append('/');
            }
            File dir = new File(rootDirectory, pathBuf.toString());
            if (dir.listFiles().length > 0) {
                break;
            } else {
                dir.delete();
                pathLength--;
            }
        }
        return true;
    }

    @Override
    public boolean createFolder(RelativePath path) {
        if (!editable) {
            throw new UnsupportedOperationException();
        }
        File dir = new File(rootDirectory, path.toString());
        if (dir.exists()) {
            return false;
        }
        String[] array = path.toArray();
        for (String name : array) {
            if (!StorageUtils.isValidDirectoryName(name)) {
                return false;
            }
        }
        dir.mkdirs();
        return true;
    }

    @Override
    public ResourceFolder getRoot() {
        if ((!rootDirectory.exists()) || (!rootDirectory.isDirectory())) {
            return StorageUtils.EMPTY_ROOTFOLDER;
        }
        return getResourceFolder(rootDirectory, "");
    }

    @Override
    public ResourceFolder getResourceFolder(RelativePath path) {
        if ((!rootDirectory.exists()) || (!rootDirectory.isDirectory())) {
            return null;
        }
        File dir = new File(rootDirectory, path.toString());
        if ((!dir.exists()) || (!dir.isDirectory())) {
            return null;
        }
        return getResourceFolder(dir, dir.getName());
    }

    private File getFile(String normalizedPath, boolean create) {
        File resourceFile = new File(rootDirectory, normalizedPath);
        if (create) {
            if ((resourceFile.exists()) && (resourceFile.isDirectory())) {
                return null;
            } else {
                return resourceFile;
            }
        } else {
            if ((resourceFile.exists()) && (!resourceFile.isDirectory())) {
                return resourceFile;
            } else {
                return null;
            }
        }
    }

    private File getBackupFile(String normalizedPath) {
        if (backupDirectory == null) {
            return null;
        }
        return new File(backupDirectory, normalizedPath);
    }

    private ResourceFolder getResourceFolder(File directory, String folderName) {
        SortedMap<String, ResourceFolder> folderMap = new TreeMap<String, ResourceFolder>();
        SortedSet<String> resourceSet = new TreeSet<String>();
        File[] files = directory.listFiles();
        int length = files.length;
        for (int i = 0; i < length; i++) {
            File f = files[i];
            String fileName = f.getName();
            if (f.isDirectory()) {
                if (StorageUtils.isValidDirectoryName(fileName)) {
                    ResourceFolder folder = getResourceFolder(f, fileName);
                    folderMap.put(fileName, folder);
                }
            } else if (!folderName.isEmpty()) {
                if (StorageUtils.isValidResourceName(fileName)) {
                    resourceSet.add(fileName);
                }
            }
        }
        return ResourceFolderBuilder.build(folderName, folderMap.values(), resourceSet);
    }

}
