/* FichothequeLib_Tools - Copyright (c) 2011-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction.builders;

import java.util.AbstractList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.RandomAccess;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.extraction.ExtractionConstants;
import net.fichotheque.extraction.def.ExtraitAddendaDef;
import net.fichotheque.extraction.def.ExtraitAlbumDef;
import net.fichotheque.extraction.def.ExtraitCorpusDef;
import net.fichotheque.extraction.def.ExtraitThesaurusDef;
import net.fichotheque.extraction.def.FicheRattachementDef;
import net.fichotheque.extraction.def.MotcleRattachementDef;
import net.fichotheque.extraction.filterunit.ChronoFilterUnit;
import net.fichotheque.extraction.filterunit.CorpsdeficheFilterUnit;
import net.fichotheque.extraction.filterunit.EnteteFilterUnit;
import net.fichotheque.extraction.filterunit.ExtraitAddendaFilterUnit;
import net.fichotheque.extraction.filterunit.ExtraitAlbumFilterUnit;
import net.fichotheque.extraction.filterunit.ExtraitCorpusFilterUnit;
import net.fichotheque.extraction.filterunit.ExtraitThesaurusFilterUnit;
import net.fichotheque.extraction.filterunit.FicheRattachementFilterUnit;
import net.fichotheque.extraction.filterunit.FieldKeyFilterUnit;
import net.fichotheque.extraction.filterunit.FieldNamePrefixFilterUnit;
import net.fichotheque.extraction.filterunit.FilterUnit;
import net.fichotheque.extraction.filterunit.IntituleFilterUnit;
import net.fichotheque.extraction.filterunit.LotFilterUnit;
import net.fichotheque.extraction.filterunit.MotcleRattachementFilterUnit;
import net.mapeadores.util.attr.Attributes;


/**
 *
 * @author Vincent Calame
 */
public final class FilterUnitUtils {

    public final static EnteteFilterUnit ENTETE_FILTERUNIT = new InternalEnteteFilterUnit();
    public final static CorpsdeficheFilterUnit CORPSDEFICHE_FILTERUNIT = new InternalCorpsdeficheFilterUnit();
    public final static ChronoFilterUnit CHRONO_FILTERUNIT = new InternalChronoFilterUnit();
    public final static IntituleFilterUnit INTITULE_FILTERUNIT = new InternalIntituleFilterUnit();
    public final static List<FilterUnit> EMPTY_FILTERUNITLIST = Collections.emptyList();

    private FilterUnitUtils() {
    }

    public static FieldKeyFilterUnit toFieldKeyFilterUnit(FieldKey fieldKey, Attributes attributes) {
        return new InternalFieldKeyFilterUnit(fieldKey, attributes);
    }

    public static LotFilterUnit toLotFilterUnit(String tagName, List<FilterUnit> filterUnitList) {
        tagName = BuilderUtils.testTagName(tagName, ExtractionConstants.LOT_DEFAULT_TAGNAME);
        return new InternalLotFilterUnit(tagName, toImmutableList(filterUnitList));
    }

    public static FieldNamePrefixFilterUnit toFieldNamePrefixFilterUnit(short category, String prefix, Attributes attributes) {
        return new InternalFieldNamePrefixFilterUnit(category, prefix, attributes);
    }

    public static ExtraitCorpusFilterUnit toExtraitCorpusFilterUnit(ExtraitCorpusDef extraitCorpusDef) {
        return new InternalExtraitCorpusFilterUnit(extraitCorpusDef);
    }

    public static ExtraitThesaurusFilterUnit toExtraitThesaurusFilterUnit(ExtraitThesaurusDef extraitThesaurusDef) {
        return new InternalExtraitThesaurusFilterUnit(extraitThesaurusDef);
    }

    public static ExtraitAlbumFilterUnit toExtraitAlbumFilterUnit(ExtraitAlbumDef extraitAlbumDef) {
        return new InternalExtraitAlbumFilterUnit(extraitAlbumDef);
    }

    public static ExtraitAddendaFilterUnit toExtraitAddendaFilterUnit(ExtraitAddendaDef extraitAlbumDef) {
        return new InternalExtraitAddendaFilterUnit(extraitAlbumDef);
    }

    public static FicheRattachementFilterUnit toFicheRattachementFilterUnit(FicheRattachementDef ficheRattachementDef) {
        return new InternalFicheRattachementFilterUnit(ficheRattachementDef);
    }

    public static MotcleRattachementFilterUnit toMotcleRattachementFilterUnit(MotcleRattachementDef motcleRattachementDef) {
        return new InternalMotcleRattachementFilterUnit(motcleRattachementDef);
    }


    private static class InternalEnteteFilterUnit implements EnteteFilterUnit {

        private InternalEnteteFilterUnit() {
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalCorpsdeficheFilterUnit implements CorpsdeficheFilterUnit {

        private InternalCorpsdeficheFilterUnit() {
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalChronoFilterUnit implements ChronoFilterUnit {

        private InternalChronoFilterUnit() {
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalIntituleFilterUnit implements IntituleFilterUnit {

        private InternalIntituleFilterUnit() {
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalFieldKeyFilterUnit implements FieldKeyFilterUnit {

        private final FieldKey fieldKey;
        private final Attributes attributes;

        private InternalFieldKeyFilterUnit(FieldKey fieldKey, Attributes attributes) {
            this.fieldKey = fieldKey;
            this.attributes = attributes;
        }

        @Override
        public FieldKey getFieldKey() {
            return fieldKey;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }


    private static class InternalLotFilterUnit implements LotFilterUnit {

        private final String tagName;
        private final List<FilterUnit> filterUnitList;

        private InternalLotFilterUnit(String tagName, List<FilterUnit> filterUnitList) {
            this.tagName = tagName;
            this.filterUnitList = filterUnitList;
        }

        @Override
        public String getTagName() {
            return tagName;
        }

        @Override
        public List<FilterUnit> getFilterUnitList() {
            return filterUnitList;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalFieldNamePrefixFilterUnit implements FieldNamePrefixFilterUnit {

        private final short category;
        private final String prefix;
        private final Attributes attributes;

        private InternalFieldNamePrefixFilterUnit(short category, String prefix, Attributes attributes) {
            this.category = category;
            this.prefix = prefix;
            this.attributes = attributes;
        }

        @Override
        public short getCategory() {
            return category;
        }

        @Override
        public String getPrefix() {
            return prefix;

        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }


    private static class InternalExtraitCorpusFilterUnit implements ExtraitCorpusFilterUnit {

        private final ExtraitCorpusDef extraitCorpusDef;

        private InternalExtraitCorpusFilterUnit(ExtraitCorpusDef extraitCorpusDef) {
            this.extraitCorpusDef = extraitCorpusDef;
        }

        @Override
        public ExtraitCorpusDef getExtraitCorpusDef() {
            return extraitCorpusDef;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalExtraitThesaurusFilterUnit implements ExtraitThesaurusFilterUnit {

        private final ExtraitThesaurusDef extraitThesaurusDef;

        private InternalExtraitThesaurusFilterUnit(ExtraitThesaurusDef extraitThesaurusDef) {
            this.extraitThesaurusDef = extraitThesaurusDef;
        }

        @Override
        public ExtraitThesaurusDef getExtraitThesaurusDef() {
            return extraitThesaurusDef;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalExtraitAlbumFilterUnit implements ExtraitAlbumFilterUnit {

        private final ExtraitAlbumDef extraitAlbumDef;

        private InternalExtraitAlbumFilterUnit(ExtraitAlbumDef extraitAlbumDef) {
            this.extraitAlbumDef = extraitAlbumDef;
        }

        @Override
        public ExtraitAlbumDef getExtraitAlbumDef() {
            return extraitAlbumDef;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalExtraitAddendaFilterUnit implements ExtraitAddendaFilterUnit {

        private final ExtraitAddendaDef extraitAddendaDef;

        private InternalExtraitAddendaFilterUnit(ExtraitAddendaDef extraitAddendaDef) {
            this.extraitAddendaDef = extraitAddendaDef;
        }

        @Override
        public ExtraitAddendaDef getExtraitAddendaDef() {
            return extraitAddendaDef;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalFicheRattachementFilterUnit implements FicheRattachementFilterUnit {

        private final FicheRattachementDef ficheRattachementDef;

        private InternalFicheRattachementFilterUnit(FicheRattachementDef ficheRattachementDef) {
            this.ficheRattachementDef = ficheRattachementDef;
        }

        @Override
        public FicheRattachementDef getFicheRattachementDef() {
            return ficheRattachementDef;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalMotcleRattachementFilterUnit implements MotcleRattachementFilterUnit {

        private final MotcleRattachementDef ficheRattachementDef;

        private InternalMotcleRattachementFilterUnit(MotcleRattachementDef ficheRattachementDef) {
            this.ficheRattachementDef = ficheRattachementDef;
        }

        @Override
        public MotcleRattachementDef getMotcleRattachementDef() {
            return ficheRattachementDef;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }

    public static List<FilterUnit> toImmutableList(Collection<FilterUnit> collection) {
        return new FilterUnitList(collection.toArray(new FilterUnit[collection.size()]));
    }

    public static List<FilterUnit> wrap(FilterUnit[] array) {
        return new FilterUnitList(array);
    }


    private static class FilterUnitList extends AbstractList<FilterUnit> implements RandomAccess {

        private final FilterUnit[] array;

        private FilterUnitList(FilterUnit[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public FilterUnit get(int index) {
            return array[index];
        }

    }

}
