/* FichothequeLib_Tools - Copyright (c) 2006-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction.dom;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import net.fichotheque.Fichotheque;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.extraction.ExtractionConstants;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.extraction.def.ExtraitAddendaDef;
import net.fichotheque.extraction.def.ExtraitAlbumDef;
import net.fichotheque.extraction.def.ExtraitCorpusDef;
import net.fichotheque.extraction.def.ExtraitThesaurusDef;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.FicheRattachementDef;
import net.fichotheque.extraction.def.GroupClause;
import net.fichotheque.extraction.def.GroupParams;
import net.fichotheque.extraction.def.IntituleClause;
import net.fichotheque.extraction.def.MotcleRattachementDef;
import net.fichotheque.extraction.def.PackClause;
import net.fichotheque.extraction.filterunit.FilterUnit;
import net.fichotheque.namespaces.BdfXmlSpace;
import net.fichotheque.selection.DocumentCondition;
import net.fichotheque.selection.DocumentQuery;
import net.fichotheque.selection.FicheCondition;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.IllustrationCondition;
import net.fichotheque.selection.IllustrationQuery;
import net.fichotheque.selection.MotcleCondition;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.tools.extraction.GroupParamsParser;
import net.fichotheque.tools.extraction.builders.BuilderUtils;
import net.fichotheque.tools.extraction.builders.DocumentFilterBuilder;
import net.fichotheque.tools.extraction.builders.ExtractionDefBuilder;
import net.fichotheque.tools.extraction.builders.ExtraitAddendaDefBuilder;
import net.fichotheque.tools.extraction.builders.ExtraitAlbumDefBuilder;
import net.fichotheque.tools.extraction.builders.ExtraitCorpusDefBuilder;
import net.fichotheque.tools.extraction.builders.ExtraitThesaurusDefBuilder;
import net.fichotheque.tools.extraction.builders.FicheFilterBuilder;
import net.fichotheque.tools.extraction.builders.FicheRattachementDefBuilder;
import net.fichotheque.tools.extraction.builders.FilterUnitListBuilder;
import net.fichotheque.tools.extraction.builders.GroupClauseBuilder;
import net.fichotheque.tools.extraction.builders.IllustrationFilterBuilder;
import net.fichotheque.tools.extraction.builders.IntituleClauseBuilder;
import net.fichotheque.tools.extraction.builders.MotcleFilterBuilder;
import net.fichotheque.tools.extraction.builders.MotcleRattachementDefBuilder;
import net.fichotheque.tools.extraction.builders.PackClauseBuilder;
import net.fichotheque.tools.selection.DocumentQueryBuilder;
import net.fichotheque.tools.selection.FicheQueryBuilder;
import net.fichotheque.tools.selection.IllustrationQueryBuilder;
import net.fichotheque.tools.selection.MotcleQueryBuilder;
import net.fichotheque.tools.selection.SelectionDOMUtils;
import net.fichotheque.utils.FichothequeUtils;
import net.fichotheque.utils.SelectionUtils;
import net.fichotheque.xml.extraction.ExtractionXMLUtils;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.attr.Attributes;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.DocumentFragmentHolder;
import net.mapeadores.util.xml.DomMessages;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class ExtractionDOMReader implements ExtractionConstants {

    private final Fichotheque fichotheque;

    public ExtractionDOMReader(Fichotheque fichotheque) {
        this.fichotheque = fichotheque;
    }

    public ExtractionDef readExtraction(Element element_xml, DocumentFragmentHolder includeCatalog) {
        NodeList includeList = element_xml.getElementsByTagName("include");
        int includeCount = includeList.getLength();
        if (includeCount > 0) {
            Element[] includeArray = new Element[includeCount];
            for (int i = 0; i < includeCount; i++) {
                includeArray[i] = (Element) includeList.item(i);
            }
            for (int i = 0; i < includeCount; i++) {
                Element includeElement = includeArray[i];
                if (includeElement.getElementsByTagName("include").getLength() > 0) {
                    //@todo Warning
                    continue;
                }
                String fragmentKey = includeElement.getAttribute("fragment-key");
                if (fragmentKey.length() > 0) {
                    DocumentFragment documentFragment = includeCatalog.getFragment(fragmentKey);
                    if (documentFragment != null) {
                        DocumentFragment importDocumentFragment = (DocumentFragment) includeElement.getOwnerDocument().importNode(documentFragment, true);
                        Node parentNode = includeElement.getParentNode();
                        parentNode.replaceChild(importDocumentFragment, includeElement);
                    } else {
                        //@todo Warning
                    }
                }
            }
        }
        ExtractionDefBuilder extractionDefBuilder = new ExtractionDefBuilder();
        String tagNameAttribute = BuilderUtils.testTagName(element_xml.getAttribute("tag-name"), EXTRACTION_DEFAULT_TAGNAME);
        ExtraitCorpusDef dynamicExtraitCorpusDef = null;
        ExtraitThesaurusDef dynamicExtraitThesaurusDef = null;
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("static")) {
                    initStaticFilterList(extractionDefBuilder, element);
                } else if ((tagname.equals("extraitcorpus")) && (dynamicExtraitCorpusDef == null)) {
                    dynamicExtraitCorpusDef = initExtraitCorpusDef(element);
                } else if ((tagname.equals("extraitthesaurus")) && (dynamicExtraitThesaurusDef == null)) {
                    dynamicExtraitThesaurusDef = initExtraitThesaurusDef(element);
                }
            }
        }
        return extractionDefBuilder
                .setTagName(tagNameAttribute)
                .setDynamicExtraitCorpusDef(dynamicExtraitCorpusDef)
                .setDynamicExtraitThesaurusDef(dynamicExtraitThesaurusDef)
                .toExtractionDef();
    }

    private void initStaticFilterList(ExtractionDefBuilder extractionDefBuilder, Element element_xml) {
        String tagNameAttribute = element_xml.getAttribute("tag-name");
        tagNameAttribute = BuilderUtils.testTagName(tagNameAttribute, STATIC_DEFAULT_TAGNAME);
        extractionDefBuilder.setStaticTagName(tagNameAttribute);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("extraitcorpus")) {
                    extractionDefBuilder.addStatic(initExtraitCorpusDef(element));
                } else if (tagname.equals("extraitthesaurus")) {
                    extractionDefBuilder.addStatic(initExtraitThesaurusDef(element));
                }
            }
        }
    }

    private FicheFilter initFicheFilter(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("fiche")) {
                    String tagNameAttribute = element.getAttribute("tag-name");
                    FilterUnitListBuilder ficheInfoListBuilder = new FilterUnitListBuilder();
                    initFilterUnitList(ficheInfoListBuilder, element);
                    return FicheFilterBuilder.init()
                            .updateFilterUnitList(ficheInfoListBuilder.toFilterUnitList())
                            .setTagName(tagNameAttribute)
                            .toFicheFilter();
                }
            }
        }
        return FicheFilterBuilder.TITRE_FICHEFILTER;
    }

    public static ExtractionDef readExtractionDef(Fichotheque fichotheque, String s, DocumentFragmentHolder documentFragmentHolder, MessageHandler messageHandler) {
        if (documentFragmentHolder == null) {
            documentFragmentHolder = XMLUtils.EMPTY_DOCUMENTFRAGMENTHOLDER;
        }
        try {
            Document document = DOMUtils.parseDocument(s);
            ExtractionDOMReader extractionDOMReader = new ExtractionDOMReader(fichotheque);
            return extractionDOMReader.readExtraction(document.getDocumentElement(), documentFragmentHolder);
        } catch (SAXException saxException) {
            DomMessages.saxException(messageHandler, saxException);
            return null;
        }
    }

    public static FicheQuery getFicheQuery(String s) {
        Set<SubsetKey> corpusKeySet = FichothequeUtils.toSubsetKeySet(SubsetKey.CATEGORY_CORPUS, s);
        if (!corpusKeySet.isEmpty()) {
            FicheQueryBuilder ficheQueryBuilder = new FicheQueryBuilder();
            ficheQueryBuilder.addCorpus(corpusKeySet);
            return ficheQueryBuilder.toFicheQuery();
        } else {
            return null;
        }
    }

    public static MotcleQuery getMotcleQuery(String s) {
        Set<SubsetKey> subsetKeySet = FichothequeUtils.toSubsetKeySet(SubsetKey.CATEGORY_THESAURUS, s);
        if (!subsetKeySet.isEmpty()) {
            return MotcleQueryBuilder.init()
                    .addThesaurus(subsetKeySet)
                    .toMotcleQuery();
        } else {
            return null;
        }
    }

    public static IllustrationQuery getIllustrationQuery(String s) {
        Set<SubsetKey> subsetKeySet = FichothequeUtils.toSubsetKeySet(SubsetKey.CATEGORY_ALBUM, s);
        if (!subsetKeySet.isEmpty()) {
            return IllustrationQueryBuilder.init()
                    .addAlbum(subsetKeySet)
                    .toIllustrationQuery();
        } else {
            return null;
        }
    }


    private void initIllustrationFilter(IllustrationFilterBuilder illustrationFilterBuilder, Element element_xml, String defaultTagName) {
        String tagNameAttribute = element_xml.getAttribute("tag-name");
        illustrationFilterBuilder.parseIllustrationTagName(tagNameAttribute, defaultTagName);
    }

    public static DocumentQuery getDocumentQuery(String s) {
        Set<SubsetKey> subsetKeySet = FichothequeUtils.toSubsetKeySet(SubsetKey.CATEGORY_ADDENDA, s);
        if (!subsetKeySet.isEmpty()) {
            return DocumentQueryBuilder.init()
                    .addAddenda(subsetKeySet)
                    .toDocumentQuery();
        } else {
            return null;
        }
    }


    private void initDocumentFilter(DocumentFilterBuilder documentFilterBuilder, Element element_xml, String defaultTagName) {
        String tagNameAttribute = element_xml.getAttribute("tag-name");
        documentFilterBuilder.parseDocumentTagName(tagNameAttribute, defaultTagName);
    }


    private void initMotcleFilter(MotcleFilterBuilder motcleFilterBuilder, Element element_xml, String defaultTagName) {
        String tagNameAttribute = element_xml.getAttribute("tag-name");
        motcleFilterBuilder.setTagName(tagNameAttribute, defaultTagName);
        if (StringUtils.isTrue(element_xml.getAttribute("recursive"))) {
            if (defaultTagName.equals(MOTCLE_DEFAULT_TAGNAME)) {
                motcleFilterBuilder.setChildrenRecursive();
            } else if (defaultTagName.equals(PARENT_DEFAULT_TAGNAME)) {
                motcleFilterBuilder.setParentRecursive();
            }
        }
        if (SelectionDOMUtils.isEmpty(element_xml)) {
            return;
        }
        FicheSelectorBundle ficheSelectorBundle = new FicheSelectorBundle();
        NodeList liste = element_xml.getChildNodes();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(j);
                String tagname = element.getTagName();
                if ((tagname.equals("labels")) || (tagname.equals("lib"))) {
                    motcleFilterBuilder.setWithLabels(true);
                } else if (tagname.equals("level")) {
                    motcleFilterBuilder.setWithLevel(true);
                } else if (tagname.equals("intitule")) {
                    motcleFilterBuilder.setWithIntitule(true);
                } else if (tagname.equals("next")) {
                    MotcleFilterBuilder nextFilterBuilder = new MotcleFilterBuilder();
                    initMotcleFilter(nextFilterBuilder, element, NEXT_DEFAULT_TAGNAME);
                    motcleFilterBuilder.setNextFilter(nextFilterBuilder.toMotcleFilter());
                } else if (tagname.equals("previous")) {
                    MotcleFilterBuilder previousFilterBuilder = new MotcleFilterBuilder();
                    initMotcleFilter(previousFilterBuilder, element, PREVIOUS_DEFAULT_TAGNAME);
                    motcleFilterBuilder.setPreviousFilter(previousFilterBuilder.toMotcleFilter());
                } else if (tagname.equals("motcle")) {
                    MotcleFilterBuilder childrenFilterBuilder = new MotcleFilterBuilder();
                    initMotcleFilter(childrenFilterBuilder, element, MOTCLE_DEFAULT_TAGNAME);
                    motcleFilterBuilder.setChildrenFilter(childrenFilterBuilder.toMotcleFilter());
                } else if (tagname.equals("parent")) {
                    MotcleFilterBuilder parentFilterBuilder = new MotcleFilterBuilder();
                    initMotcleFilter(parentFilterBuilder, element, PARENT_DEFAULT_TAGNAME);
                    motcleFilterBuilder.setParentFilter(parentFilterBuilder.toMotcleFilter());
                } else if (tagname.equals("fiche")) {
                    FicheRattachementDef ficheRattachementDef = initFicheRattachementDef(element);
                    motcleFilterBuilder.addFicheRattachementDef(ficheRattachementDef);
                } else {
                    ficheSelectorBundle.readExtraitCorpus(element);
                }
            }
        }
        if (ficheSelectorBundle.isNotEmpty()) {
            for (ExtraitCorpusDef extraitCorpusDef : ficheSelectorBundle.list) {
                motcleFilterBuilder.addIndexationExtraitCorpusDef(extraitCorpusDef);
            }
        }
    }

    private void initFilterUnitList(FilterUnitListBuilder builder, Element element_xml) {
        if (SelectionDOMUtils.isEmpty(element_xml)) {
            return;
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                testFilterUnit(builder, element);
            }
        }
    }

    private void testFilterUnit(FilterUnitListBuilder builder, Element element) {
        String paramsString = element.getAttribute("params");
        Attributes paramsAttributes = null;
        if (paramsString.length() > 0) {
            paramsString = paramsString.replace("libtype", "labeltype");
            paramsAttributes = AttributeUtils.parseInstruction(paramsString, BdfXmlSpace.EXTRACTION_NAMESPACE, null);
        }
        String tagname = element.getTagName();
        if (tagname.equals("entete")) {
            builder.addEnteteFilter();
        } else if (tagname.equals("corpsdefiche")) {
            builder.addCorpsdeficheFilter();
        } else if (tagname.equals("titre")) {
            builder.addFieldKeyFilter(FieldKey.TITRE, paramsAttributes);
        } else if (tagname.equals("soustitre")) {
            builder.addFieldKeyFilter(FieldKey.SOUSTITRE, paramsAttributes);
        } else if (tagname.equals("propriete")) {
            String fieldName = element.getAttribute("name");
            if (fieldName.length() == 0) {
                fieldName = element.getAttribute("type");
            }
            if (fieldName.length() > 0) {
                builder.addFieldNameFilter(FieldKey.PROPRIETE_CATEGORY, fieldName, paramsAttributes);
            }
        } else if (tagname.equals("redacteurs")) {
            builder.addFieldKeyFilter(FieldKey.REDACTEURS, paramsAttributes);
        } else if (tagname.equals("information")) {
            String fieldName = element.getAttribute("name");
            if (fieldName.length() == 0) {
                fieldName = element.getAttribute("type");
            }
            if (fieldName.length() > 0) {
                builder.addFieldNameFilter(FieldKey.INFORMATION_CATEGORY, fieldName, paramsAttributes);
            }
        } else if (tagname.equals("section")) {
            String fieldName = element.getAttribute("name");
            if (fieldName.length() == 0) {
                fieldName = element.getAttribute("type");
            }
            if (fieldName.length() > 0) {
                builder.addFieldNameFilter(FieldKey.SECTION_CATEGORY, fieldName, paramsAttributes);
            }
        } else if (tagname.equals("lot")) {
            String tagName = element.getAttribute("tag-name");
            FilterUnitListBuilder builder2 = new FilterUnitListBuilder();
            initFilterUnitList(builder2, element);
            List<FilterUnit> filterUnitList = builder2.toFilterUnitList();
            if (!filterUnitList.isEmpty()) {
                builder.addLotFilter(tagName, filterUnitList);
            }
        } else if (tagname.equals("lang")) {
            builder.addFieldKeyFilter(FieldKey.LANG, paramsAttributes);
        } else if (tagname.equals("chrono")) {
            builder.addChronoFilter();
        } else if (tagname.equals("extraitaddenda")) {
            ExtraitAddendaDef extraitAddendaDef = initExtraitAddendaDef(element);
            builder.addExtraitAddendaFilter(extraitAddendaDef);
        } else if (tagname.equals("extraitcorpus")) {
            ExtraitCorpusDef extraitCorpusDef = initExtraitCorpusDef(element);
            builder.addExtraitCorpusFilter(extraitCorpusDef);
        } else if (tagname.equals("extraitthesaurus")) {
            ExtraitThesaurusDef extraitThesaurusDef = initExtraitThesaurusDef(element);
            builder.addExtraitThesaurusFilter(extraitThesaurusDef);
        } else if (tagname.equals("extraitalbum")) {
            ExtraitAlbumDef extraitAlbumDef = initExtraitAlbumDef(element);
            builder.addExtraitAlbumFilter(extraitAlbumDef);
        } else if (tagname.equals("intitule")) {
            builder.addIntituleFilter();
        } else if (tagname.equals("fiche")) {
            FicheRattachementDef ficheRattachementDef = initFicheRattachementDef(element);
            builder.addFicheRattachementFilter(ficheRattachementDef);
        } else if (tagname.equals("motcle")) {
            MotcleRattachementDef motcleRattachementDef = initMotcleRattachementDef(element);
            builder.addMotcleRattachementFilter(motcleRattachementDef);
        }
    }


    private class FicheSelectorBundle implements ElementHandler {

        private final List<ExtraitCorpusDef> list = new ArrayList<ExtraitCorpusDef>();

        private FicheSelectorBundle() {
        }

        private boolean isNotEmpty() {
            return (!list.isEmpty());
        }

        @Override
        public void readElement(Element element) {
            readExtraitCorpus(element);
        }

        private boolean readExtraitCorpus(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("extraitcorpus")) {
                ExtraitCorpusDef extraitCorpusDef = initExtraitCorpusDef(element);
                list.add(extraitCorpusDef);
                return true;
            }
            return false;
        }

    }

    private ExtraitAddendaDef initExtraitAddendaDef(Element element_xml) {
        String extraitName = element_xml.getAttribute("extrait-name");
        String customTagName = element_xml.getAttribute("tag-name");
        DocumentCondition.Entry attributeEntry = null;
        String addendaAttribute = element_xml.getAttribute("addenda");
        if (addendaAttribute.length() > 0) {
            DocumentQuery attributeQuery = getDocumentQuery(addendaAttribute);
            if (attributeQuery != null) {
                attributeEntry = SelectionUtils.toDocumentConditionEntry(attributeQuery);
            }
        }
        DocumentFilterBuilder documentFilterBuilder = new DocumentFilterBuilder();
        NodeList nodeList = element_xml.getChildNodes();
        List<DocumentCondition.Entry> queryList = new ArrayList<DocumentCondition.Entry>();
        for (int i = 0; i < nodeList.getLength(); i++) {
            if (nodeList.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) nodeList.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("document")) {
                    initDocumentFilter(documentFilterBuilder, element, DOCUMENT_DEFAULT_TAGNAME);
                } else if (tagname.equals("document-select")) {
                    queryList.add(SelectionDOMUtils.getDocumentConditionEntry(element));
                }
            }
        }
        if ((queryList.isEmpty()) && (attributeEntry != null)) {
            queryList.add(attributeEntry);
        }
        return ExtraitAddendaDefBuilder.init(documentFilterBuilder.toDocumentFilter())
                .setExtraitName(extraitName)
                .setTagName(customTagName)
                .setEntryList(queryList)
                .toExtraitAddendaDef();
    }

    private ExtraitAlbumDef initExtraitAlbumDef(Element element_xml) {
        String extraitName = element_xml.getAttribute("extrait-name");
        String customTagName = element_xml.getAttribute("tag-name");
        IllustrationCondition.Entry attributeEntry = null;
        String albumAttribute = element_xml.getAttribute("album");
        if (albumAttribute.length() > 0) {
            IllustrationQuery attributeQuery = getIllustrationQuery(albumAttribute);
            if (attributeQuery != null) {
                attributeEntry = SelectionUtils.toIllustrationConditionEntry(attributeQuery);
            }
        }
        IllustrationFilterBuilder illustrationFilterBuilder = new IllustrationFilterBuilder();
        NodeList nodeList = element_xml.getChildNodes();
        List<IllustrationCondition.Entry> entryList = new ArrayList<IllustrationCondition.Entry>();
        for (int i = 0; i < nodeList.getLength(); i++) {
            if (nodeList.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) nodeList.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("illustration")) {
                    initIllustrationFilter(illustrationFilterBuilder, element, ILLUSTRATION_DEFAULT_TAGNAME);
                } else if (tagname.equals("illustration-select")) {
                    entryList.add(SelectionDOMUtils.getIllustrationConditionEntry(element));
                }
            }
        }
        if ((entryList.isEmpty()) && (attributeEntry != null)) {
            entryList.add(attributeEntry);
        }
        return ExtraitAlbumDefBuilder.init(illustrationFilterBuilder.toIllustrationFilter())
                .setExtraitName(extraitName)
                .setTagName(customTagName)
                .setEntryList(entryList)
                .toExtraitAlbumDef();
    }

    private ExtraitThesaurusDef initExtraitThesaurusDef(Element element_xml) {
        String extraitName = element_xml.getAttribute("extrait-name");
        String customTagName = element_xml.getAttribute("tag-name");
        boolean master = false;
        String sourceAttribute = element_xml.getAttribute("source");
        if (sourceAttribute.equals("master")) {
            master = true;
        }
        MotcleCondition.Entry attributeEntry = null;
        String thesaurusAttribute = element_xml.getAttribute("thesaurus");
        if (thesaurusAttribute.length() > 0) {
            MotcleQuery attributeQuery = getMotcleQuery(thesaurusAttribute);
            if (attributeQuery != null) {
                attributeEntry = SelectionUtils.toMotcleConditionEntry(attributeQuery);
            }
        }
        MotcleFilterBuilder motcleFilterBuilder = new MotcleFilterBuilder();
        NodeList liste = element_xml.getChildNodes();
        List<MotcleCondition.Entry> entryList = new ArrayList<MotcleCondition.Entry>();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("motcle")) {
                    initMotcleFilter(motcleFilterBuilder, element, MOTCLE_DEFAULT_TAGNAME);
                } else if ((tagname.equals("motcle-query")) || (tagname.equals("motcle-select"))) {
                    entryList.add(SelectionDOMUtils.getMotcleConditionEntry(fichotheque, element));
                } else if (tagname.equals("intitule")) {
                    motcleFilterBuilder.setWithThesaurusTitle(true);
                }
            }
        }
        if ((entryList.isEmpty()) && (attributeEntry != null)) {
            entryList.add(attributeEntry);
        }
        return ExtraitThesaurusDefBuilder.init(motcleFilterBuilder.toMotcleFilter())
                .setExtraitName(extraitName)
                .setTagName(customTagName)
                .setEntryList(entryList)
                .setMaster(master)
                .toExtraitThesaurusDef();
    }

    private ExtraitCorpusDef initExtraitCorpusDef(Element element_xml) {
        String extraitName = element_xml.getAttribute("extrait-name");
        String customTagName = element_xml.getAttribute("tag-name");
        boolean master = false;
        String sourceAttribute = element_xml.getAttribute("source");
        if (sourceAttribute.equals("master")) {
            master = true;
        }
        FicheCondition.Entry attributeEntry = null;
        String corpusAttribute = element_xml.getAttribute("corpus");
        if (corpusAttribute.length() > 0) {
            FicheQuery attributeQuery = getFicheQuery(corpusAttribute);
            if (attributeQuery != null) {
                attributeEntry = SelectionUtils.toFicheConditionEntry(attributeQuery);
            }
        }
        GroupClause attributeGroupClause = null;
        String sortAttribute = element_xml.getAttribute("sort");
        if (sortAttribute.length() > 0) {
            attributeGroupClause = GroupClauseBuilder.getGroupClause(sortAttribute, true);
        }
        boolean descendantAxis = false;
        String axisAttribute = element_xml.getAttribute("axis");
        if (axisAttribute.equals("descendant")) {
            descendantAxis = true;
        }
        GroupClause groupClause = null;
        PackClause packClause = null;
        IntituleClause intituleClause = null;
        List<FicheCondition.Entry> entryList = new ArrayList<FicheCondition.Entry>();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("group")) {
                    groupClause = initGroupClause(element);
                } else if ((tagname.equals("fiche-query")) || (tagname.equals("fiche-select"))) {
                    entryList.add(SelectionDOMUtils.getFicheConditionEntry(fichotheque, element));
                } else if (tagname.equals("intitule")) {
                    intituleClause = initIntituleClause(element);
                } else if (tagname.equals("pack")) {
                    packClause = initPackClause(element);
                }
            }
        }
        if ((entryList.isEmpty()) && (attributeEntry != null)) {
            entryList.add(attributeEntry);
        }
        if (groupClause == null) {
            groupClause = attributeGroupClause;
        }
        FicheFilter ficheFilter = initFicheFilter(element_xml);
        return ExtraitCorpusDefBuilder.init(ficheFilter)
                .setExtraitName(extraitName)
                .setTagName(customTagName)
                .setEntryList(entryList)
                .setMaster(master)
                .setGroupClause(groupClause)
                .setPackClause(packClause)
                .setIntituleClause(intituleClause)
                .setDescendantAxis(descendantAxis)
                .toExtraitCorpusDef();
    }

    private static PackClause initPackClause(Element element_xml) {
        int packSize;
        try {
            packSize = Integer.parseInt(element_xml.getAttribute("size"));
        } catch (NumberFormatException nfe) {
            return null;
        }
        int modulo = 0;
        try {
            modulo = Integer.parseInt(element_xml.getAttribute("modulo"));
        } catch (NumberFormatException nfe) {
        }
        String tagName = element_xml.getAttribute("tag-name");
        return PackClauseBuilder.init(packSize)
                .setTagName(tagName)
                .setModulo(modulo)
                .toPackClause();
    }

    private static IntituleClause initIntituleClause(Element element_xml) {
        return IntituleClauseBuilder.DEFAULT;
    }

    private static GroupClause initGroupClause(Element element_xml) {
        short groupType;
        try {
            groupType = ExtractionXMLUtils.typeToShort(element_xml.getAttribute("type"));
        } catch (IllegalArgumentException iae) {
            return null;
        }
        short sortOrder = ExtractionConstants.ASCENDING_SORT;
        try {
            sortOrder = ExtractionXMLUtils.sortToShort(element_xml.getAttribute("sort"));
        } catch (IllegalArgumentException iae) {
        }
        String params = element_xml.getAttribute("params");
        if (params.length() == 0) {
            params = element_xml.getAttribute("mode");
        }
        GroupParams groupParams = GroupParamsParser.parse(groupType, params);
        String tagName = element_xml.getAttribute("tag-name");
        GroupClause subGroupClause = null;
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("group")) {
                    subGroupClause = initGroupClause(element);
                }
            }
        }
        try {
            return GroupClauseBuilder.init(groupType)
                    .setTagName(tagName)
                    .setSortOrder(sortOrder)
                    .setGroupParams(groupParams)
                    .setSubGroupClause(subGroupClause)
                    .toGroupClause();
        } catch (IllegalStateException iae) {
            return null;
        }
    }

    private FicheRattachementDef initFicheRattachementDef(Element element_xml) {
        String tagNameAttribute = element_xml.getAttribute("tag-name");
        FilterUnitListBuilder ficheInfoListBuilder = new FilterUnitListBuilder();
        initFilterUnitList(ficheInfoListBuilder, element_xml);
        FicheFilter ficheFilter = FicheFilterBuilder.init()
                .updateFilterUnitList(ficheInfoListBuilder.toFilterUnitList())
                .setTagName(tagNameAttribute)
                .toFicheFilter();
        FicheRattachementDefBuilder ficheRattachementDefBuilder = new FicheRattachementDefBuilder(ficheFilter);
        String corpusAttribute = element_xml.getAttribute("corpus");
        if (corpusAttribute.length() > 0) {
            Set<SubsetKey> corpusKeySet = FichothequeUtils.toSubsetKeySet(SubsetKey.CATEGORY_CORPUS, corpusAttribute);
            if (!corpusKeySet.isEmpty()) {
                for (SubsetKey corpusKey : corpusKeySet) {
                    ficheRattachementDefBuilder.addRattachementSubsetKey(corpusKey);
                }
            }
        }
        return ficheRattachementDefBuilder.toFicheRattachementDef();
    }

    private MotcleRattachementDef initMotcleRattachementDef(Element element_xml) {
        MotcleFilterBuilder filterBuilder = new MotcleFilterBuilder();
        initMotcleFilter(filterBuilder, element_xml, MOTCLE_DEFAULT_TAGNAME);
        return MotcleRattachementDefBuilder
                .init(filterBuilder.toMotcleFilter())
                .toMotcleRattachementDef();
    }

}
