/* FichothequeLib_Tools - Copyright (c) 2021-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.selection;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.fichotheque.Fichotheque;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.selection.CroisementCondition;
import net.fichotheque.selection.DocumentCondition;
import net.fichotheque.selection.FicheCondition;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.FieldQuery;
import net.fichotheque.selection.IllustrationCondition;
import net.fichotheque.selection.MotcleCondition;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.selection.PeriodCondition;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.sphere.Sphere;
import net.fichotheque.utils.FichothequeUtils;
import net.fichotheque.utils.SelectionUtils;
import net.fichotheque.utils.SphereUtils;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionsConstants;
import net.mapeadores.util.conditions.ConditionsUtils;
import net.mapeadores.util.conditions.ConditionsXMLStorage;
import net.mapeadores.util.primitives.FuzzyDate;
import net.mapeadores.util.primitives.RangeUtils;
import net.mapeadores.util.primitives.Ranges;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;


/**
 *
 * @author Vincent Calame
 */
public final class SelectionDOMUtils {

    private SelectionDOMUtils() {

    }

    public static ElementHandler getQueryHandler(Fichotheque fichotheque, FichothequeQueryBuilder fichothequeQueryBuilder) {
        return new QueryHandler(fichotheque, fichothequeQueryBuilder);
    }

    public static boolean readQueryElement(FichothequeQueryBuilder fichothequeQueryBuilder, Fichotheque fichotheque, Element element) {
        switch (element.getTagName()) {
            case "fiche-query":
            case "fiche-select":
                FicheCondition.Entry ficheEntry = getFicheConditionEntry(fichotheque, element);
                fichothequeQueryBuilder.addFicheQuery(ficheEntry.getFicheQuery());
                return true;
            case "motcle-query":
            case "motcle-select":
                MotcleCondition.Entry motcleEntry = getMotcleConditionEntry(fichotheque, element);
                fichothequeQueryBuilder.addMotcleQuery(motcleEntry.getMotcleQuery());
                return true;
            default:
                return false;
        }
    }

    public static FicheCondition.Entry getFicheConditionEntry(Fichotheque fichotheque, Element element) {
        if (isEmpty(element)) {
            return SelectionUtils.EMPTY_FICHECONDITIONENTRY;
        }
        FicheQueryBuilder ficheQueryBuilder = new FicheQueryBuilder();
        StringBuilder corpusNameBuffer = new StringBuilder();
        CroisementConditionBuilder croisementConditionBuilder = new CroisementConditionBuilder();
        FicheQueryHandler ficheQueryHandler = new FicheQueryHandler(fichotheque, ficheQueryBuilder, croisementConditionBuilder, corpusNameBuffer);
        DOMUtils.readChildren(element, ficheQueryHandler);
        if (corpusNameBuffer.length() > 0) {
            Set<SubsetKey> corpusKeySet = FichothequeUtils.toSubsetKeySet(SubsetKey.CATEGORY_CORPUS, corpusNameBuffer.toString());
            if (!corpusKeySet.isEmpty()) {
                ficheQueryBuilder.addCorpus(corpusKeySet);
            }
        }
        CroisementCondition croisementCondition;
        if (!croisementConditionBuilder.isEmpty()) {
            croisementCondition = croisementConditionBuilder.toCroisementCondition();
        } else {
            croisementCondition = null;
        }
        return SelectionUtils.toFicheConditionEntry(ficheQueryBuilder.toFicheQuery(), croisementCondition);
    }

    private static void initRedacteurList(Fichotheque fichotheque, Element element_xml, FicheQueryBuilder ficheQueryBuilder) {
        List<String> redacteurGlobalIdList = new ArrayList<String>();
        List<SubsetKey> subsetKeyList = new ArrayList<SubsetKey>();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("redacteur")) {
                    String sphereName = element.getAttribute("sphere");
                    if (sphereName.length() > 0) {
                        try {
                            SubsetKey sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, sphereName);
                            Sphere sphere = (Sphere) fichotheque.getSubset(sphereKey);
                            if (sphere != null) {
                                Redacteur redacteur = null;
                                String login = element.getAttribute("login");
                                if (login.length() == 0) {
                                    login = element.getAttribute("idsphere");
                                }
                                if (login.length() > 0) {
                                    redacteur = sphere.getRedacteurByLogin(login);
                                } else {
                                    String idString = element.getAttribute("id");
                                    if (idString.length() == 0) {
                                        idString = element.getAttribute("idsph");
                                    }
                                    try {
                                        int id = Integer.parseInt(idString);
                                        redacteur = sphere.getRedacteurById(id);
                                    } catch (NumberFormatException nfe) {
                                    }
                                }
                                if (redacteur != null) {
                                    redacteurGlobalIdList.add(redacteur.getGlobalId());
                                }
                            }

                        } catch (ParseException pe) {
                        }
                    } else {
                        String data = XMLUtils.getData(element);
                        if (data.length() > 0) {
                            try {
                                Redacteur redacteur = SphereUtils.parse(fichotheque, data);
                                redacteurGlobalIdList.add(redacteur.getGlobalId());
                            } catch (SphereUtils.RedacteurLoginException rle) {

                            }
                        }
                    }
                } else if (tagname.equals("sphere")) {
                    String sphereName = XMLUtils.getData(element);
                    if (sphereName.length() > 0) {
                        try {
                            SubsetKey sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, sphereName);
                            if (fichotheque.getSubset(sphereKey) != null) {
                                subsetKeyList.add(sphereKey);
                            }
                        } catch (ParseException pe) {
                        }
                    }
                }
            }
        }
        if ((!redacteurGlobalIdList.isEmpty()) || (!subsetKeyList.isEmpty())) {
            ficheQueryBuilder.setRedacteurFilter(redacteurGlobalIdList, subsetKeyList);
        } else {
            ficheQueryBuilder.setRedacteurFilter(FicheQuery.ALL_REDACTEURFILTER);
        }
    }

    private static Set<FieldKey> getFieldKeySet(Element element_xml) {
        Set<FieldKey> fieldKeySet = new LinkedHashSet<FieldKey>();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("field")) {
                    try {
                        FieldKey fieldKey = FieldKey.parse(XMLUtils.getData(element));
                        fieldKeySet.add(fieldKey);
                    } catch (ParseException pe) {
                    }
                }
            }
        }
        return fieldKeySet;
    }

    private static PeriodCondition getPeriodCondition(Element element) {
        String startDateString = element.getAttribute("start");
        if (startDateString.length() == 0) {
            return null;
        }
        PeriodConditionBuilder builder = new PeriodConditionBuilder();
        if (startDateString.equals(PeriodCondition.ANY_CHAR)) {
            builder.setAnyStartDate();
        } else {
            try {
                FuzzyDate startDate = FuzzyDate.parse(startDateString);
                builder.setStartDate(startDate);
            } catch (ParseException pe) {
                return null;
            }
        }
        String endDateString = element.getAttribute("end");
        if (endDateString.length() > 0) {
            if (endDateString.equals(PeriodCondition.ANY_CHAR)) {
                builder.setAnyEndDate();
            } else {
                try {
                    FuzzyDate endDate = FuzzyDate.parse(endDateString);
                    builder.setEndDate(endDate);
                } catch (ParseException pe) {
                }
            }
        }
        NodeList nodeList = element.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            if (nodeList.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element child = (Element) nodeList.item(i);
                String tagname = child.getTagName();
                if (tagname.equals("field")) {
                    try {
                        FieldKey fieldKey = FieldKey.parse(XMLUtils.getData(child));
                        builder.addFieldKey(fieldKey);
                    } catch (ParseException pe) {
                    }
                } else if (tagname.equals("chrono")) {
                    String chrono = XMLUtils.getData(child);
                    if (chrono.equals("creation")) {
                        builder.setOnCreationDate(true);
                    } else if (chrono.equals("modification")) {
                        builder.setOnModificationDate(true);
                    }
                }
            }
        }
        return builder.toPeriodCondition();
    }

    public static IllustrationCondition.Entry getIllustrationConditionEntry(Element element_xml) {
        if (isEmpty(element_xml)) {
            return SelectionUtils.EMPTY_ILLUSTRATIONCONDITIONENTRY;
        }
        IllustrationQueryBuilder illustrationQueryBuilder = new IllustrationQueryBuilder();
        CroisementConditionBuilder croisementConditionBuilder = new CroisementConditionBuilder();
        StringBuilder albumNameBuffer = new StringBuilder();
        NodeList nodeList = element_xml.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            if (nodeList.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) nodeList.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("album")) {
                    String albumName = XMLUtils.getData(element);
                    if (albumName.length() > 0) {
                        if (albumNameBuffer.length() > 0) {
                            albumNameBuffer.append(';');
                        }
                        albumNameBuffer.append(albumName);
                    }
                } else if (tagname.equals("albumdim")) {
                    String albumDimName = XMLUtils.getData(element);
                    if (albumDimName.length() > 0) {
                        illustrationQueryBuilder.addAlbumDimName(albumDimName);
                    }
                } else if (testConditionCroisement(element, croisementConditionBuilder)) {

                }
            }
        }
        if (albumNameBuffer.length() > 0) {
            illustrationQueryBuilder.addAlbum(FichothequeUtils.toSubsetKeySet(SubsetKey.CATEGORY_ALBUM, albumNameBuffer.toString()));
        }
        CroisementCondition croisementCondition;
        if (!croisementConditionBuilder.isEmpty()) {
            croisementCondition = croisementConditionBuilder.toCroisementCondition();
        } else {
            croisementCondition = null;
        }
        return SelectionUtils.toIllustrationConditionEntry(illustrationQueryBuilder.toIllustrationQuery(), croisementCondition);
    }

    public static MotcleCondition.Entry getMotcleConditionEntry(Fichotheque fichotheque, Element element) {
        if (isEmpty(element)) {
            return SelectionUtils.EMPTY_MOTCLECONDITIONENTRY;
        }
        MotcleQueryBuilder motcleQueryBuilder = new MotcleQueryBuilder();
        CroisementConditionBuilder croisementConditionBuilder = new CroisementConditionBuilder();
        StringBuilder thesaurusNameBuffer = new StringBuilder();
        MotcleQueryHandler handler = new MotcleQueryHandler(fichotheque, motcleQueryBuilder, croisementConditionBuilder, thesaurusNameBuffer);
        DOMUtils.readChildren(element, handler);
        CroisementCondition croisementCondition;
        if (!croisementConditionBuilder.isEmpty()) {
            croisementCondition = croisementConditionBuilder.toCroisementCondition();
        } else {
            croisementCondition = null;
        }
        if (thesaurusNameBuffer.length() > 0) {
            Set<SubsetKey> subsetKeySet = FichothequeUtils.toSubsetKeySet(SubsetKey.CATEGORY_THESAURUS, thesaurusNameBuffer.toString());
            if (!subsetKeySet.isEmpty()) {
                motcleQueryBuilder.addThesaurus(subsetKeySet);
            }
        }
        return SelectionUtils.toMotcleConditionEntry(motcleQueryBuilder.toMotcleQuery(), croisementCondition);
    }

    public static DocumentCondition.Entry getDocumentConditionEntry(Element element_xml) {
        if (isEmpty(element_xml)) {
            return SelectionUtils.EMPTY_DOCUMENTCONDITIONENTRY;
        }
        DocumentQueryBuilder documentQueryBuilder = new DocumentQueryBuilder();
        CroisementConditionBuilder croisementConditionBuilder = new CroisementConditionBuilder();
        StringBuilder addendaNameBuffer = new StringBuilder();
        NodeList nodeList = element_xml.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            if (nodeList.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) nodeList.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("addenda")) {
                    String addendaName = XMLUtils.getData(element);
                    if (addendaName.length() > 0) {
                        if (addendaNameBuffer.length() > 0) {
                            addendaNameBuffer.append(';');
                        }
                        addendaNameBuffer.append(addendaName);
                    }
                } else if (testConditionCroisement(element, croisementConditionBuilder)) {
                }
            }
        }
        if (addendaNameBuffer.length() > 0) {
            documentQueryBuilder.addAddenda(FichothequeUtils.toSubsetKeySet(SubsetKey.CATEGORY_ADDENDA, addendaNameBuffer.toString()));
        }
        CroisementCondition croisementCondition;
        if (!croisementConditionBuilder.isEmpty()) {
            croisementCondition = croisementConditionBuilder.toCroisementCondition();
        } else {
            croisementCondition = null;
        }
        return SelectionUtils.toDocumentConditionEntry(documentQueryBuilder.toDocumentQuery(), croisementCondition);
    }

    private static boolean testConditionCroisement(Element element, CroisementConditionBuilder builder) {
        String tagName = element.getTagName();
        if ((tagName.equals("lien-poids")) || (tagName.equals("poids"))) {
            String range = XMLUtils.getData(element);
            if (range.length() > 0) {
                Ranges ranges = RangeUtils.positiveRangeParse(range);
                if (ranges != null) {
                    boolean isExclude = false;
                    String mode = element.getAttribute("mode");
                    if (mode.equals("exclude")) {
                        isExclude = true;
                    }
                    builder.setPoidsRanges(ranges, isExclude);
                }
            }
            return true;
        } else if (tagName.equals("lien-mode")) {
            builder.addLienMode(XMLUtils.getData(element));
            return true;
        }
        return false;
    }

    private static short getSelectMode(Element element_xml) {
        String modeString = element_xml.getAttribute("mode");
        if (modeString.equals("and")) {
            return ConditionsConstants.MODE_AND;
        } else if (modeString.equals("or")) {
            return ConditionsConstants.MODE_OR;
        } else {
            return 0;
        }
    }

    private static Condition initFieldCondition(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (ConditionsXMLStorage.isConditionElement(tagname)) {
                    return ConditionsXMLStorage.getCondition(element);
                }
            }
        }
        return null;
    }

    public static boolean isEmpty(Element element) {
        int size = element.getChildNodes().getLength();
        if (size == 0) {
            return true;
        }
        if (size > 1) {
            return false;
        }
        Node node = element.getFirstChild();
        if (node.getNodeType() != Node.TEXT_NODE) {
            return false;
        }
        String data = ((Text) node).getData().trim();
        return data.isEmpty();
    }


    private static class QueryHandler implements ElementHandler {

        private final Fichotheque fichotheque;
        private final FichothequeQueryBuilder fichothequeQueryBuilder;

        private QueryHandler(Fichotheque fichotheque, FichothequeQueryBuilder fichothequeQueryBuilder) {
            this.fichotheque = fichotheque;
            this.fichothequeQueryBuilder = fichothequeQueryBuilder;
        }

        @Override
        public void readElement(Element element) {
            readQueryElement(fichothequeQueryBuilder, fichotheque, element);
        }

    }


    private static class FicheQueryHandler implements ElementHandler {

        private final Fichotheque fichotheque;
        private final FicheQueryBuilder ficheQueryBuilder;
        private final CroisementConditionBuilder croisementConditionBuilder;
        private final StringBuilder corpusNameBuilder;

        private FicheQueryHandler(Fichotheque fichotheque, FicheQueryBuilder ficheQueryBuilder, CroisementConditionBuilder croisementConditionBuilder, StringBuilder corpusNameBuilder) {
            this.fichotheque = fichotheque;
            this.ficheQueryBuilder = ficheQueryBuilder;
            this.croisementConditionBuilder = croisementConditionBuilder;
            this.corpusNameBuilder = corpusNameBuilder;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            switch (tagName) {
                case "corpus": {
                    String corpusName = XMLUtils.getData(element);
                    if (corpusName.length() > 0) {
                        if (corpusNameBuilder.length() > 0) {
                            corpusNameBuilder.append(';');
                        }
                        corpusNameBuilder.append(corpusName);
                    }
                    break;
                }
                case "range": {
                    String range = XMLUtils.getData(element);
                    if (range.length() > 0) {
                        Ranges ranges = RangeUtils.positiveRangeParse(range);
                        ficheQueryBuilder.setIdRanges(ranges);
                    }
                    break;
                }
                case "discard": {
                    try {
                        ficheQueryBuilder.setDiscardFilter(element.getAttribute("filter"));
                    } catch (IllegalArgumentException iae) {

                    }
                    break;
                }
                case "field-query":
                case "field-select": {
                    Condition fieldCondition = initFieldCondition(element);
                    if ((fieldCondition != null) && (!ConditionsUtils.isNeutral(fieldCondition))) {
                        short fieldConditionScope = FieldQuery.TITRE_SCOPE;
                        String scopeString = element.getAttribute("scope");
                        try {
                            fieldConditionScope = SelectionUtils.fieldConditionToShort(scopeString);
                        } catch (IllegalArgumentException iae) {
                        }
                        Set<FieldKey> fieldKeySet = null;
                        if (fieldConditionScope == FieldQuery.SELECTION_SCOPE) {
                            fieldKeySet = getFieldKeySet(element);
                            if (fieldKeySet.isEmpty()) {
                                fieldConditionScope = FieldQuery.TITRE_SCOPE;
                            }
                        }
                        ficheQueryBuilder.setFieldCondition(fieldCondition, fieldConditionScope, fieldKeySet);
                    }
                    break;
                }
                case "motcle-query":
                case "motcle-select": {
                    MotcleCondition.Entry motcleEntry = getMotcleConditionEntry(fichotheque, element);
                    ficheQueryBuilder.addMotcleConditionEntry(motcleEntry);
                    break;
                }
                case "motcle-query-mode":
                case "motcle-select-mode": {
                    short mode = getSelectMode(element);
                    if (mode != 0) {
                        ficheQueryBuilder.setMotcleSelectMode(mode);
                    }
                    break;
                }
                case "redacteurs": {
                    String redacteurFilterString = element.getAttribute("filter");
                    String redacteurFilter = FicheQuery.SOME_REDACTEURFILTER;
                    if (!redacteurFilterString.isEmpty()) {
                        try {
                            redacteurFilter = SelectionUtils.checkRedacteurFilter(redacteurFilterString);
                        } catch (IllegalArgumentException iae) {

                        }
                    }
                    if (redacteurFilter.equals(FicheQuery.SOME_REDACTEURFILTER)) {
                        initRedacteurList(fichotheque, element, ficheQueryBuilder);
                    } else {
                        ficheQueryBuilder.setRedacteurFilter(redacteurFilter);
                    }
                    break;
                }
                case "period": {
                    ficheQueryBuilder.setPeriodCondition(getPeriodCondition(element));
                    break;
                }
                case "satellites":
                case "rattachement": {
                    ficheQueryBuilder.setWithRattachement(true);
                    break;
                }
                case "current": {
                    ficheQueryBuilder.setWithCurrentCorpus(true);
                    break;
                }
                case "geoloc": {
                    ficheQueryBuilder.setWithGeoloc(true);
                    break;
                }
                default: {
                    testConditionCroisement(element, croisementConditionBuilder);
                }
            }
        }

    }


    private static class MotcleQueryHandler implements ElementHandler {

        private final Fichotheque fichotheque;
        private final MotcleQueryBuilder motcleQueryBuilder;
        private final CroisementConditionBuilder croisementConditionBuilder;
        private final StringBuilder thesaurusNameBuffer;

        private MotcleQueryHandler(Fichotheque fichotheque, MotcleQueryBuilder motcleQueryBuilder, CroisementConditionBuilder croisementConditionBuilder, StringBuilder thesaurusNameBuffer) {
            this.fichotheque = fichotheque;
            this.motcleQueryBuilder = motcleQueryBuilder;
            this.croisementConditionBuilder = croisementConditionBuilder;
            this.thesaurusNameBuffer = thesaurusNameBuffer;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            switch (tagName) {
                case "thesaurus": {
                    String thesaurusName = XMLUtils.getData(element);
                    if (thesaurusName.length() > 0) {
                        if (thesaurusNameBuffer.length() > 0) {
                            thesaurusNameBuffer.append(';');
                        }
                        thesaurusNameBuffer.append(thesaurusName);
                    }
                    break;
                }
                case "range": {
                    String range = XMLUtils.getData(element);
                    if (range.length() > 0) {
                        Ranges ranges = RangeUtils.positiveRangeParse(range);
                        motcleQueryBuilder.setIdRangeList(ranges);
                    }
                    break;
                }
                case "level": {
                    String range = XMLUtils.getData(element);
                    if (range.length() > 0) {
                        Ranges ranges = RangeUtils.positiveRangeParse(range);
                        motcleQueryBuilder.setLevelRangeList(ranges);
                    }
                    break;
                }
                case "satellites":
                case "rattachement": {
                    motcleQueryBuilder.setIncludeSatellites(true);
                    break;
                }
                case "fiche-query":
                case "fiche-select": {
                    FicheCondition.Entry ficheEntry = getFicheConditionEntry(fichotheque, element);
                    motcleQueryBuilder.addFicheConditionQuery(ficheEntry);
                    break;
                }
                case "fiche-query-mode":
                case "fiche-select-mode": {
                    short mode = getSelectMode(element);
                    if (mode != 0) {
                        motcleQueryBuilder.setFicheSelectMode(mode);
                    }
                    break;
                }
                default: {
                    if (ConditionsXMLStorage.isConditionElement(tagName)) {
                        short scope = MotcleQuery.ONLY_IDALPHA_SCOPE;
                        try {
                            scope = SelectionUtils.motcleFieldConditionScopeToShort(element.getAttribute("scope"));
                        } catch (IllegalArgumentException iae) {

                        }
                        Condition condition = ConditionsXMLStorage.getCondition(element);
                        if (!ConditionsUtils.isNeutral(condition)) {
                            motcleQueryBuilder.setContentCondition(condition, scope);
                        }
                    } else {
                        testConditionCroisement(element, croisementConditionBuilder);
                    }
                }
            }
        }

    }


}
